library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

use work.axi3intercon_pkg.all;
use work.axi_r_router_pkg.all;

entity axi3_intercon_r_router is
	port(
		clk         : in  std_logic;
		rst         : in  std_logic;
		slaves_in   : out axi_r_slaves_in_t(0 to SLAVE_COUNT);
		slaves_out  : in  axi_r_slaves_out_t(0 to SLAVE_COUNT);
		masters_in  : out axi_r_masters_in_t(0 to MASTER_COUNT - 1);
		masters_out : in  axi_r_masters_out_t(0 to MASTER_COUNT - 1)
	);
end entity axi3_intercon_r_router;

architecture RTL of axi3_intercon_r_router is
	type master_indexes_t is array (natural range <>) of integer range 0 to MASTER_COUNT - 1;
	type r_state_t is (R_READY, R_ACTIVE);
	type r_states_t is array (0 to SLAVE_COUNT) of r_state_t;
	signal r_states : r_states_t;
begin
	r_router : process(clk, rst) is
		variable master_in_use : std_logic_vector(0 to MASTER_COUNT - 1) := (others => '0');
		variable master_idx    : master_indexes_t(0 to SLAVE_COUNT);
	begin
		if rst = '1' then
			master_in_use := (others => '0');
			for i in 0 to SLAVE_COUNT loop
				slaves_in(i).rready <= '0';

				r_states(i) <= R_READY;
			end loop;
			for i in 0 to MASTER_COUNT - 1 loop
				masters_in(i).rvalid <= '0';
			end loop;
		-- TODO: Reset
		elsif rising_edge(clk) then
			for i in 0 to SLAVE_COUNT loop
				case r_states(i) is
					when R_READY =>
						if slaves_out(i).rvalid = '1' then
							-- calculate master	
							master_idx(i) := to_integer(unsigned(slaves_out(i).rid(RID_SLAVE_BITS - 1 downto RID_MASTER_BITS)));
							if (master_in_use(master_idx(i)) = '0') then
								master_in_use(master_idx(i))     := '1';
								slaves_in(i).rready              <= '1';
								masters_in(master_idx(i)).rid    <= slaves_out(i).rid(masters_in(master_idx(i)).rid'range);
								masters_in(master_idx(i)).rdata  <= slaves_out(i).rdata;
								masters_in(master_idx(i)).rlast  <= slaves_out(i).rlast;
								masters_in(master_idx(i)).rresp  <= slaves_out(i).rresp;
								masters_in(master_idx(i)).ruser  <= slaves_out(i).ruser;
								masters_in(master_idx(i)).rvalid <= '1';
								r_states(i)                      <= R_ACTIVE;
							end if;
						end if;
					when R_ACTIVE =>
						slaves_in(i).rready <= '0';
						if masters_out(master_idx(i)).rready = '1' then
							masters_in(master_idx(i)).rvalid <= '0';
							master_in_use(master_idx(i))     := '0';
							r_states(i)                      <= R_READY;
						end if;
				end case;
			end loop;
		end if;
	end process r_router;

end architecture RTL;
